<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class FloaPayEligibilityModuleFrontController extends ModuleFrontController
{
    const INTERNATIONAL_ZIP = '99';
    
    /**
     * initContent
     *
     * @return void
     */
    public function initContent()
    {
        parent::initContent();
    }
    
    /**
     * postProcess
     *
     * @return void
     */
    public function postProcess()
    {
        header('Content-Type: application/json');
        $method           = Tools::getValue('method');
        $customer         = $this->context->customer;
        $cart             = $this->context->cart;
        if (Tools::getToken() != Tools::getValue('token')) {
            $this->eligibilityResponse(
                500,
                false,
                [$this->module->l('Your session is no longer valid, please refresh the page', 'floapay')]
            );
        }
        if (!array_key_exists($method, floaPay::getPaymentMethodsAvailable())) {
            $this->eligibilityResponse(
                500,
                false,
                [$this->module->l('The payment method is no longer available', 'floapay')]
            );
        }
        if (!$customer->isLogged() && !$customer->isGuest()) {
            $this->eligibilityResponse(
                500,
                false,
                [$this->module->l('Your session is no longer valid, please refresh the page', 'floapay')]
            );
        }
        $floaPayApi         = new floaPayApi($method);
        $totalCartAmount    = $cart->getOrderTotal(true, Cart::BOTH);
        $shippingAddress    = new Address($cart->id_address_delivery);
        $billingAddress     = new Address($cart->id_address_invoice);
        $formErrors         = false;
        $isPaymentDateValid = true;
        $customer_country       = Country::getIsoById($shippingAddress->id_country);
        $score_civility         = Tools::getValue('score_civility');
        $score_ddn              = Tools::getValue('score_ddn');
        $score_cp               = Tools::getValue('score_cp');
        $score_maidenname       = Tools::getValue('score_maidenname', null);
        $score_nif              = Tools::getValue('score_nif', null);
        $score_secondlastname   = Tools::getValue('score_secondlastname', null);
        if ($score_maidenname) {
            $score_maidenname = str_replace("'","",str_replace("-",'',$score_maidenname));
        }
        $score_phone      = Tools::getValue('score_phone');
        $payment_date     = Tools::getValue('payment_date') != null ? Tools::getValue('payment_date') : false;
        $regexDate = "/^((((19|[2-9]\d)\d{2})\-(0[13578]|1[02])\-".
            "(0[1-9]|[12]\d|3[01]))|(((19|[2-9]\d)\d{2})\-(0[13456".
            "789]|1[012])\-(0[1-9]|[12]\d|30))|(((19|[2-9]\d)\d{2})".
            "\-02\-(0[1-9]|1\d|2[0-8]))|(((1[6-9]|[2-9]\d)(0[48]|[2468]".
            "[048]|[13579][26])|((16|[2468][048]|[3579][26])00))\-02\-29))$/";
        $regexMaidenName     = "/^[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]+(([',. -][a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ ])?[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]*)*$/";
        $regexSecondLastName = "/^[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]+(([',. -][a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ ])?[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]*)*$/";
        switch ($customer_country) {
            case 'FR' :
            default :
                $regexPhone = "/^((\+)33|0|0033)[1-9](\d{2}){4}$/";
                $regexCp = "/^\d{2}|^\d{3}$|^\d{5}$/";
                $isSecondLastNameValid = true;
                $isScoreMaidenNameValid = $score_civility == 2 ? preg_match($regexMaidenName, $score_maidenname) : true;
                $isNifValid = true;
                break;
            case 'IT' : 
                $regexPhone = "/^((?:0039)?|(?:\+39)?)3[0-9]{8,9}$/";
                $regexCp = "/^99$|^\d{3}$|^\d{5}$/";
                $regexNif = "/^([a-zA-Z]{6})+([0-9]{2})+([a-zA-Z]{1})+([0-9]{2})+([a-zA-Z]{1})+([0-9]{3})+([a-zA-Z]{1})$/";
                $isSecondLastNameValid = true;
                $isScoreMaidenNameValid = true;
                $isNifValid = preg_match($regexNif, $score_nif);
                break;
            case 'BE' : 
                $regexPhone = "/^((\+|00)32\s?|0)4(\d{2})(\s?\d{2}){3}$/";
                $regexCp = "/^99$|^\d{4}$/";
                $isSecondLastNameValid = true;
                $isScoreMaidenNameValid = $score_civility == 2 ? preg_match($regexMaidenName, $score_maidenname) : true;
                $isNifValid = true;
                break;
            case 'PT' : 
                $regexPhone = "/^((\+351|00351|351)?)(2\d{1}|(9(3|6|2|1)))\d{7}$/";
                $regexCp = "/^99$|^\d{4}-\d{3}$/";
                $regexNif = "/^[0-9]{9}$/";
                $isSecondLastNameValid = true;
                $isScoreMaidenNameValid = true;
                $isNifValid = preg_match($regexNif, $score_nif);
                break;
            case 'ES' : 
                $regexPhone = "/^(([+]|00)34)?(?:[0-9]{8,20})$/";
                $regexCp = "/^99$|^\d{5}$/";
                $regexNif = "/^([a-z]|[A-Z]|[0-9])[0-9]{7}([a-z]|[A-Z]|[0-9])$/";
                $isScoreMaidenNameValid = true;
                $isSecondLastNameValid = $score_secondlastname != "" ? preg_match($regexSecondLastName, $score_secondlastname) : true;
                $isNifValid = preg_match($regexNif, $score_nif);
                if ($isNifValid) {
                    $isNifValid = $this->isEsNifValid($score_nif, $score_cp, $billingAddress->postcode);
                }
                break;
            }
        $isScoreDdnValid = preg_match($regexDate, $score_ddn);
        $isScoreCivilityValid = in_array($score_civility, array(1,2));
        $isScoreCpValid = preg_match($regexCp, $score_cp);
        $isScorePhoneValid = preg_match($regexPhone, $score_phone);
        $scoreDdnObject = new DateTime($score_ddn);
        $isMajorCustomer = ($scoreDdnObject->diff(new DateTime())->y >= 18) ? true : false;
        $reportDelayInDays = 0;
        if (($method == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 1)
            && (
                !preg_match($regexDate, $payment_date)
                || $payment_date > date('Y-m-d', strtotime('+30days'))
                || $payment_date <= date('Y-m-d')
            )) {
                $isPaymentDateValid = false;
        }
        $errors = [];
        if (!$isScoreCivilityValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Civility is invalid.', 'floapay');
        }
        if (!$isSecondLastNameValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Second lastname is invalid.', 'floapay');
        }
        if (!$isScoreDdnValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Birth date is invalid.', 'floapay');
        }
        if (!$isScoreCpValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Postal code is invalid.', 'floapay');
        }
        if (!$isNifValid) {
            $formErrors = true;
            $errors[] = $this->module->l('NIF is invalid.', 'floapay');
        }
        if (!$isScorePhoneValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Phone number is invalid.', 'floapay');
        }
        if (!$isMajorCustomer) {
            $formErrors = true;
            $errors[] = $this->module->l('You must be over 18 years old.', 'floapay');
        }
        if (!$isScoreMaidenNameValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Maiden name is invalid.', 'floapay');
        }
        if (!$isPaymentDateValid) {
            $formErrors = true;
            $errors[] = $this->module->l('Payment date is invalid.', 'floapay');
        }
        if ($method == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 1) {
            $reportDelayInDays = (strtotime($payment_date)-strtotime(date('Y-m-d')))/86400;
        } elseif ($method == "CB1XD") {
            $reportDelayInDays = FloaPay::getConfKey('FLOA_CB1XD_REPORT_DELAY');
        }
        if ($formErrors) {
            $this->eligibilityResponse(
                500,
                false,
                $errors
            );
        }
        if (!$floaPayApi->active
        || $floaPayApi->contextCheck['state'] != true
        || $totalCartAmount > $floaPayApi->maxAmount
        || $totalCartAmount < $floaPayApi->minAmount) {
            floaLogger::error($floaPayApi->contextCheck['message']);
            $this->eligibilityResponse(
                500,
                false,
                [$this->module->l('The payment method is no longer available', 'floapay')]
            );
        }
        $eligibilityDatas = [
            'score_civility' => $score_civility,
            'score_ddn' => $score_ddn,
            'score_nif' => $score_nif,
            'score_cp' => $score_cp,
            'method' => $method,
            'score_maidenname' => $score_maidenname,
            'score_secondlastname' => $score_secondlastname,
            'score_phone' => $score_phone,
            'payment_date' => $payment_date,
            'customer_email' => $this->context->customer->email,
            'customer_id' => $this->context->customer->id,
            'customer_country' => $customer_country

        ];
        $eligibility = $floaPayApi->getEligibility(
            $cart,
            $shippingAddress,
            $billingAddress,
            $eligibilityDatas,
            $reportDelayInDays
        );
        if (!$eligibility['state']) {
            $this->eligibilityResponse(
                200,
                false,
                [
                    sprintf($this->module->l('Refusal reason %s%s', 'floapay'), '<br />', '<br />'),
                ]
            );
        }
        if ($method == "CB10X") {
            $url = false;
            if (isset($eligibility['datas']['links'])) {
                foreach ($eligibility['datas']['links'] as $link) {
                    if ($link->rel == "continue-long-term-consumer-credit-process") {
                        $url = $link->href;
                    }
                }
            }
            if ($url) {
                $floaOrder = new floaOrder;
                $floaOrder->merchant_id         = $floaPayApi->merchantId;
                $floaOrder->merchant_site_id    = $floaPayApi->merchantSiteId;
                $floaOrder->id_cart             = $cart->id;
                $floaOrder->scoring_date        = date('Y-m-d H:i:s');
                $floaOrder->scoring_token       = $eligibility['datas']['token'];
                $floaOrder->request_id          = $eligibility['datas']['requestId'];
                $floaOrder->op                  = $method;
                $floaOrder->order_reference     = $eligibility['datas']['orderRef'];
                $floaOrder->total_amount        = $eligibility['datas']['totalAmount'];
                $floaOrder->fees_amount         = $eligibility['datas']['totalAmount']-floaTools::convertToInt(
                    $cart->getOrderTotal(true, Cart::BOTH)
                );
                $floaOrder->save();
                $this->lockCartPayment();
                $this->eligibilityResponse(
                    200,
                    true,
                    ['success'],
                    $url
                );
            } else {
                floaLogger::error('The payment method is no longer available');
                $this->eligibilityResponse(
                    500,
                    false,
                    [$this->module->l('The payment method is no longer available', 'floapay')]
                );
            }
        } else {
            $paymentSession = $floaPayApi->createPaymentSession(
                $eligibilityDatas['customer_country'],
                $cart,
                $shippingAddress,
                $billingAddress,
                $eligibility['datas'],
                $reportDelayInDays
            );
            if (!$paymentSession['state'] || ! $paymentSession['session']->paymentSessionUrl) {
                floaLogger::error('The payment method is no longer available');
                $this->eligibilityResponse(
                    500,
                    false,
                    [$this->module->l('The payment method is no longer available', 'floapay')]
                );
            }
            $floaOrder = new floaOrder;
            $floaOrder->merchant_id         = $floaPayApi->merchantId;
            $floaOrder->merchant_site_id    = $floaPayApi->merchantSiteId;
            $floaOrder->id_cart             = $cart->id;
            $floaOrder->scoring_date        = date('Y-m-d H:i:s');
            $floaOrder->scoring_token       = $eligibility['datas']['token'];
            $floaOrder->payment_session_id  = $paymentSession['session']->paymentSessionId;
            $floaOrder->request_id          = $eligibility['datas']['requestId'];
            $floaOrder->op                  = $method;
            $floaOrder->order_reference     = $eligibility['datas']['orderRef'];
            $floaOrder->total_amount        = $eligibility['datas']['totalAmount'];
            $floaOrder->fees_amount         = (($eligibility['datas']['totalAmount'])-floaTools::convertToInt(
                $cart->getOrderTotal(true, Cart::BOTH)
            ));
            $floaOrder->save();
            $this->lockCartPayment();
            $this->eligibilityResponse(200, true, ['success'], $paymentSession['session']->paymentSessionUrl);
        }
    }

    protected function isEsNifValid($identificationNumber, $birthZip, $billingZip)
    {
        if (preg_match("/^K[0-9]{7}[A-Z]{1}$/", $identificationNumber)) {
            return false;
        } else if (preg_match("/^L[0-9]{7}[A-Z]{1}$/", $identificationNumber)) {
            if ($billingZip != self::INTERNATIONAL_ZIP || $birthZip == self::INTERNATIONAL_ZIP) {
                return false;
            }
        } else if (preg_match("/^[0-9]{8}[A-Z]{1}$|^[ILMOTXYZ][0-9]{7}[A-Z]{1}$/", $identificationNumber)) {
            if ($billingZip == self::INTERNATIONAL_ZIP) {
                return false;
            }
        } else {
            return false;
        }

        return true;
    }

    public function eligibilityResponse($httpcode, $success, $messages = [], $url = '')
    {
        header('X-PHP-Response-Code: '.$httpcode, true, $httpcode);
        $response = Tools::JsonEncode(
            array(
                'success'  => $success,
                'messages' => $messages,
                'url'      => $url
            )
        );
        if (method_exists(get_parent_class($this), 'ajaxRender')) {
            parent::ajaxRender($response);
            exit;
        } elseif (method_exists(get_parent_class($this), 'ajaxDie')) {
            parent::ajaxDie($response);
        } else {
            die($response);
        }
    }
    
    /**
     * lockCartPayment
     *
     * Lock Cart for preventing editing during payment session
     *
     * @return bool
     */
    public function lockCartPayment()
    {
        $this->context->cart = false;
        $this->context->cookie->id_cart = 0;
        return true;
    }
}
