<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class paymentValidation
{
    
    /**
     * __construct
     *
     * @param  Context $context
     * @param  Module $module
     * @return void
     */
    public function __construct($context, $module)
    {
        $this->context = $context;
        $this->module = $module;
    }
    
    /**
     * validatePayment
     *
     * Validate Order and Payment from API response
     * 
     * @param  string $orderRef
     * @param  string $scoringToken
     * @param  integer $ipn
     * @return void
     */
    public function validatePayment($orderRef, $scoringToken, $returnCode, $ipn = false)
    {
        $error          = false;
        $floaOrder      = floaOrder::getByReference($orderRef);
        $scoringToken   = $scoringToken;
        $floaOrderId    = false;
        if (!isset($floaOrder['order_reference'])
            || !$orderRef
            || $scoringToken != $floaOrder['scoring_token']
        ) {
            $error = true;
            floaLogger::info('['.$orderRef.'] Order not found');
        } else {
            if ($floaOrder['id_order'] != 0) {
                $cart = new Cart($floaOrder['id_cart']);
                $customer = new Customer($cart->id_customer);
                if (!$ipn) {
                    $customer->logged = 1;
                    $this->context->customer = $customer;
                    $this->context->cookie->id_customer = (int) $customer->id;
                    $this->context->cookie->customer_lastname = $customer->lastname;
                    $this->context->cookie->customer_firstname = $customer->firstname;
                    $this->context->cookie->logged = 1;
                    $this->context->cookie->is_guest = $customer->isGuest();
                }
                $floaOrderId = $floaOrder['id_order'];
            } else {
                $floaPayApi         = new floaPayApi($floaOrder['op']);
                if ($floaOrder['op'] == "CB10X") {
                    $paymentSessionID   = "CCL_".$scoringToken;
                    $paymentInformation = $floaPayApi->getPaymentResultFromPaymentSchedules($floaOrder['order_reference']);
                } else {
                    $paymentSessionID   = $floaOrder['payment_session_id'];
                    $paymentInformation = $floaPayApi->getPaymentResult($paymentSessionID);
                }
                if ($floaPayApi->active) {
                    if (isset($paymentInformation['state']) && $paymentInformation['state'] == true) {
                        $cart = new Cart($floaOrder['id_cart']);
                        if ($cart && $cart->id_customer != 0 && $cart->id_address_delivery != 0 && $cart->id_address_invoice != 0) {
                            $customer = new Customer($cart->id_customer);
                            if (!$ipn) {
                                $customer->logged = 1;
                                $this->context->customer = $customer;
                                $this->context->cookie->id_customer = (int) $customer->id;
                                $this->context->cookie->customer_lastname = $customer->lastname;
                                $this->context->cookie->customer_firstname = $customer->firstname;
                                $this->context->cookie->logged = 1;
                                $this->context->cookie->is_guest = $customer->isGuest();
                            }
                            if (Validate::isLoadedObject($customer)) {
                                if (!$cart->OrderExists()) {
                                    $totalAmountPaid = Tools::ps_round((floaTools::convertToFloat($paymentInformation["datas"]['amount'])-floaTools::convertToFloat($floaOrder['fees_amount'])), 2);
                                    if ($totalAmountPaid == $cart->getOrderTotal(true, Cart::BOTH)) {
                                        if ($paymentInformation['datas']["paymentResultCode"] === 0 || $paymentInformation['datas']["paymentResultCode"] === 4) {
                                            $floaOrderUpdate = new floaOrder($floaOrder['id']);
                                            if ($this->module->validateOrder(
                                                (int) $cart->id,
                                                Configuration::get('PS_OS_PAYMENT'),
                                                $totalAmountPaid,
                                                $this->module->displayName,
                                                null,
                                                ['transaction_id' => $paymentSessionID],
                                                (int) $cart->id_currency,
                                                false,
                                                $customer->secure_key
                                            )) {
                                                $order = $this->getOrderByCartId((int) $cart->id);
                                                $floaOrderId = $order->id;
                                                $floaOrderUpdate->payment_date = date('Y-m-d H:i:s');
                                                $floaOrderUpdate->id_order = $floaOrderId;
                                                $floaOrderUpdate->save();
                                            } else {
                                                floaLogger::error('['.$orderRef.'] Unable to create PrestaShop Order');
                                                $error = true;
                                            }
                                        }
                                    } else {
                                        floaLogger::error('['.$orderRef.'] Purchase amount mismatch');
                                        $error = true;
                                    }
                                } else {
                                    floaLogger::error('['.$orderRef.'] Order aleready exist for this Cart');
                                    $error = true;
                                }
                            } else {
                                floaLogger::error('['.$orderRef.'] Unable to retrieve customer');
                                $error = true;
                            }
                        } else {
                            floaLogger::error('['.$orderRef.'] Unable to retrieve customer Cart');
                            $error = true;
                        }
                    } else {
                        if ($floaOrder['op'] != "CB10X" && $returnCode != 2) {
                            floaLogger::error('['.$orderRef.'] Unable to retrieve payment Session');
                            $error = true;
                        }
                    }
                } else {
                    floaLogger::error('['.$orderRef.'] Payment method is no longer available');
                    $error = true;
                }
            }
        }

        if (!$error) {
            if ($floaOrderId) {
                $link = 'id_cart='.(int)$cart->id.
                    '&id_module='.(int)$this->module->id.
                    '&id_order='.(int)$floaOrderId.
                    '&key='.$customer->secure_key;
                return ['success' => true, 'orderId' => $floaOrderId, 'link' => $link];
            } else {
                if (!$ipn) {
                    if ($cart) {
                        $newCart = $this->cloneCart($cart);
                        $this->context->cart = $newCart;
                        $this->context->cart->update();
                        $this->context->cookie->id_cart = (int)$newCart->id;
                    }
                }
                return ['success' => false, 'type' => 'refused'];
            }
        } else {
            return ['success' => false, 'type' => 'error'];
        }
    }
    
    /**
     * getOrderByCartId
     *
     * Get an order from is Cart ID
     * 
     * @param  integer $cartId
     * @return Order
     */
    private function getOrderByCartId($cartId)
    {
        if (is_callable(['Order', 'getByCartId'])) {
            return Order::getByCartId((int) $cartId);
        } else {
            $orderId = (int) Order::getOrderByCartId((int) $cartId);
            return new Order($orderId);
        }
    }
    
    /**
     * cloneCart
     *
     * Clone a Cart
     * 
     * @param  Cart $cart
     * @return Cart
     */
    public function cloneCart($cart)
    {
        $old_cart_secure_key = $cart->secure_key;
        $old_cart_customer_id = (int)$cart->id_customer;
        $cart_products = $cart->getProducts();
        $newCart = new Cart();
        $newCart->id_lang = $this->context->language->id;
        $newCart->id_currency = $this->context->currency->id;
        $newCart->add();
        foreach ($cart_products as $product) {
            $newCart->updateQty((int) $product['quantity'], (int) $product['id_product'], (int) $product['id_product_attribute']);
        }
        $newCart->id_customer = $old_cart_customer_id;
        $newCart->secure_key = $old_cart_secure_key;
        $newCart->save();
        return $newCart;
    }
}
