<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class eligibilityHelper
{
    
    /**
     * customer
     *
     * @param  Address $billingAddress
     * @param  Address $eligibilityDatas
     * @return array
     */
    public static function customer($billingAddress, $eligibilityDatas)
    {
        $orderDates = self::getMinAndMaxOrderDate($eligibilityDatas['customer_id']);
        return [
            "civility"          => ($eligibilityDatas['score_civility'] == 1) ? 'Mr' : ($eligibilityDatas['score_civility'] == 2 ? 'Mrs' : 'Mr'),
            "lastName"          => $billingAddress->lastname,
            "firstName"         => $billingAddress->firstname,
            "birthName"         => $eligibilityDatas['score_maidenname'],
            "SecondLastName"    => $eligibilityDatas['score_secondlastname'],
            "cellPhoneNumber"   => $eligibilityDatas['score_phone'],
            "reference"         => $eligibilityDatas['customer_id'],
            "birthDate"         => $eligibilityDatas['score_ddn'],
            "birthZipCode"      => $eligibilityDatas['score_cp'],
            "email"             => $eligibilityDatas['customer_email'],
            "homeAddress"       => 
            [
                "line1"         => $billingAddress->address1,
                "line2"         => $billingAddress->address2,
                "zipCode"       => self::formatPostCode(Country::getIsoById($billingAddress->id_country), $billingAddress->postcode),
                "city"          => $billingAddress->city,
                "countryCode"   => Country::getIsoById($billingAddress->id_country),
            ],
            "history" => [
                "firstOrderDate"    => $orderDates['first'],
                "lastOrderDate"     => $orderDates['last']
            ],
            "trustLevel"    => "Standard",
            "ipAddress"     => floaTools::getUserIP(),
            "nationalId"    => $eligibilityDatas['score_nif']
        ];
    }
    
    /**
     * shoppingCarts
     *
     * @param  Cart $cart
     * @param  Address $shippingAddress
     * @param  Address $orderRef
     * @param  string $shippingMethod
     * @return array
     */
    public static function shoppingCarts($cart, $shippingAddress, $orderRef, $shippingMethod)
    {
        $productsList = $cart->getProducts();
        $products     = array();
        $productCount = 0;
        if (isset($productsList) && count($productsList) > 0) {
            foreach ($productsList as $product) {
                $iso = Country::getIsoById($shippingAddress->id_country);
                $products[] = [
                    "name"          => $product['name'],
                    "rawAmount"     => floaTools::convertToInt($product['total']),
                    "categories"    => [
                        [
                            "name"      => $product['category'],
                            "parent"    => ""
                        ]
                    ],
                    "shipping" => [
                        "method"        => $shippingMethod ? $shippingMethod : "STD",
                        "delayInDays"   => 0,
                        "address"       => [
                            "line1"         => $shippingAddress->address1,
                            "line2"         => $shippingAddress->address2,
                            "zipCode"       => self::formatPostCode($iso, $shippingAddress->postcode),
                            "city"          => $shippingAddress->city,
                            "countryCode"   => $iso,
                                        ]
                    ]
                ];
                $productCount += $product['quantity'];
            }
        }
        return [
            [
                "reference"     => $orderRef,
                "rawAmount"     => floaTools::convertToInt($cart->getOrderTotal(true, Cart::BOTH)),
                "productsCount" => $productCount,
                "products"      => $products
            ]
        ];
    }
    
    /**
     * saleChannel
     *
     * @return array
     */
    public static function saleChannel()
    {
        try {
            if (Tools::version_compare(_PS_VERSION_, '1.6', '<')) {
                require_once(_PS_TOOL_DIR_ . 'mobile_Detect/Mobile_Detect.php');
                $mobile_detect = new Mobile_Detect();
            } else {
                $mobile_detect = Context::getContext()->getMobileDetect();
            }
            if ($mobile_detect->isTablet()) {
                if ($mobile_detect->is('ipad')) {
                    $result = 'Ipad';
                } else {
                    $result = 'Tablet';
                }
            } elseif ($mobile_detect->isMobile()) {
                if ($mobile_detect->is('iphone')) {
                    $result = 'IPhone';
                } elseif ($mobile_detect->is('androidos')) {
                    $result = 'AndroidSmartPhone';
                } else {
                    $result = 'SmartPhone';
                }
            }
        } catch (Exception $e) {
            return 'Desktop';
        }

        return isset($result) ? $result : 'Desktop';
    }
    
    /**
     * merchantSite
     *
     * @param  Cart $cart
     * @return array
     */
    public static function merchantSite($cart)
    {
        $context = Context::getContext();
        return [
            "homeUrl"           => _PS_BASE_URL_ . __PS_BASE_URI__,
            "backUrl"           => $context->link->getModuleLink('floapay', 'back', [
                'goto'  => @$_SERVER['HTTP_REFERER'],
                'ref'   => $cart->id,
                'token' => Tools::getToken()
            ], true),
            "returnUrl"         => $context->link->getModuleLink('floapay', 'return', [], true),
            "notificationUrl"   => $context->link->getModuleLink('floapay', 'ipn', [], true),
        ];
    }
    
    /**
     * address
     *
     * @param  Address $address
     * @return array
     */
    public static function address($address)
    {
        $iso = Country::getIsoById($address->id_country);
        return [
            "line1"         => $address->address1,
            "line2"         => $address->address2,
            "zipCode"       => self::formatPostCode($iso, $address->postcode),
            "city"          => $address->city,
            "countryCode"   => $iso,
        ];
    }

    public static function formatPostCode($iso, $postCode)
    {
        switch($iso) {
            case 'PT':
                return preg_replace('/\ +/', '-', $postCode);
            default:
                return $postCode;
        }
    }
    
    /**
     * getMinAndMaxOrderDate
     *
     * @param  integer $id
     * @return array
     */
    public static function getMinAndMaxOrderDate($id)
    {
        $sql = 'SELECT 
                  DATE_FORMAT(MIN(`date_add`), "%Y-%m-%d") as first, 
                  DATE_FORMAT(MAX(`date_add`), "%Y-%m-%d") as last 
                FROM `' . _DB_PREFIX_ . 'orders`
                WHERE id_customer=' . (int) $id;
        return Db::getInstance()->getRow($sql);
    }

}