<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class paymentSessionHelper
{
    
    /**
     * customer
     *
     * @param  Address $billingAddress
     * @param  Address $shippingAddress
     * @param  integer $customerId
     * @return array
     */
    public static function customer($billingAddress, $shippingAddress, $customerId, $country)
    {
        return [
            "billingAddress" => [
                "city"      => $billingAddress->city,
                "line1"     => $billingAddress->address1,
                "name"      => "Billing address",
                "zipCode"   => $billingAddress->postcode,
            ],
             "deliveryAddress" => [
                "city"      => $shippingAddress->city,
                "line1"     => $shippingAddress->address1,
                "name"      => "Shipping address",
                "zipCode"   => $shippingAddress->postcode,
             ],
             "customerRef"  => $customerId,
             "country"      => $country
        ];
    }
    
    /**
     * configuration
     *
     * @param  string $country
     * @param  Cart $cart
     * @param  integer $paymentOptionRef
     * @param  integer $reportDelayInDays
     * @return array
     */
    public static function configuration($country, $cart, $paymentOptionRef, $reportDelayInDays = 0)
    {
        $context = Context::getContext();

        return [
            "culture"                   => FloaTools::computeCulture($country),
            "merchantHomeUrl"           => _PS_BASE_URL_ . __PS_BASE_URI__,
            "merchantBackUrl"           => $context->link->getModuleLink('floapay', 'back', [
                'goto'  => @$_SERVER['HTTP_REFERER'],
                'ref'   => $cart->id,
                'token' => Tools::getToken()
            ], true),
            "merchantReturnUrl"         => $context->link->getModuleLink('floapay', 'return', [], true),
            "merchantNotifyUrl"         => $context->link->getModuleLink('floapay', 'ipn', [], true),
            "paymentOptionRef"          => $paymentOptionRef,
            "formType"                  => "default",
            "reportDelayInDays"         => $reportDelayInDays,
            "country"                   => $country
        ];
    }
    
    /**
     * orderData
     *
     * @param  Address $shippingAddress
     * @param  Address $scoringDatas
     * @param  Cart $cart
     * @return array
     */
    public static function orderData($shippingAddress, $scoringDatas, $cart)
    {
        return [
            "shippingAddress" => [
                "city"      => $shippingAddress->city,
                "line1"     => $shippingAddress->address1,
                "name"      => "Shipping address",
                "zipCode"   => $shippingAddress->postcode,
            ],
            "orderDiscountAmount"   => floaTools::convertToInt($cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS)),
            "orderFeesAmount"       => $scoringDatas['totalAmount']-floaTools::convertToInt($cart->getOrderTotal(true, Cart::BOTH)),
            "orderRowsAmount"       => floaTools::convertToInt($cart->getOrderTotal(true, Cart::ONLY_PRODUCTS)),
            "orderShippingAmount"   => floaTools::convertToInt(( version_compare(_PS_VERSION_, '1.6', '<') 
            ? $cart->getTotalShippingCost() : $cart->getPackageShippingCost())),
            "orderSummaryRef"       => $scoringDatas['orderRef'],
            "orderRef"              => $scoringDatas['orderRef'],
            "amount"                => $scoringDatas['totalAmount'],
            "scoringToken"          => $scoringDatas['token'],
        ];
    }

}