<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class hookDisplayPaymentFloa {
    
    /**
     * __construct
     *
     * @return void
     */
    public function __construct()
    {
        $this->module = Module::getInstanceByName("floapay");
    }
    
    /**
     * run
     *
     * @param  string $payment_key
     * @param  integer $ech
     * @param  Cart $cart
     * @param  integer $reportDelayInDays
     * @return array
     */
    public function run($payment_key, $ech, $cart, $reportDelayInDays)
    {
        $invoiceAddress     = new Address($cart->id_address_invoice);
        $paymentCountryCode = FloaTools::getCountryCode(Country::getIsoById($invoiceAddress->id_country));
        $availablesCulture  = $this->module->allowedCultureByCountry($paymentCountryCode);
        $generatedCulture   = FloaTools::computeCulture($paymentCountryCode);
        $floaPayApi         = new floaPayApi($payment_key);
        $totalCartAmount    = $cart->getOrderTotal(true, Cart::BOTH);
        $displayPayment     = false;
        if ($floaPayApi->active 
            && $floaPayApi->contextCheck['state'] == true
            && $totalCartAmount <= $floaPayApi->maxAmount
            && $totalCartAmount >= $floaPayApi->minAmount
            && $paymentCountryCode == FloaPay::getConfKey('FLOA_MERCHANT_COUNTRY')
            && $availablesCulture[$generatedCulture] == true
        )
        {
            $getEstimatedSchedule = $floaPayApi->getEstimatedSchedule($totalCartAmount, $reportDelayInDays);
            if (isset($getEstimatedSchedule['schedules']))
            {                
                $reportDelay = 0;
                $reportMode  = 0;
                $callToActionText = sprintf($this->module->l('Pay in %s instalment%s', 'floapay'), $ech, ($ech > 1 ? 's' : ''));
                if ($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 1)
                {
                    $callToActionText .= ' '.$this->module->l('deferred up to 30 days', 'floapay');
                    $reportMode = FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE');
                } else if($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 2) {
                    $callToActionText .= ' '.sprintf($this->module->l('deferred for %s days', 'floapay'), FloaPay::getConfKey('FLOA_CB1XD_REPORT_DELAY'));
                    $reportDelay = FloaPay::getConfKey('FLOA_CB1XD_REPORT_DELAY');
                    $reportMode = FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE');
                }
                $callToActionText .= ' '.$this->module->l('with floa', 'floapay');
                $displayPayment = array(
                    'callToActionText'   => $callToActionText,
                    'method'             => $payment_key,
                    'schedules'          => $getEstimatedSchedule['schedules'],
                    'fees'               => (floaTools::convertToFloat($getEstimatedSchedule['amount'])-$totalCartAmount),
                    'totalPlan'          => floaTools::convertToFloat($getEstimatedSchedule['amount']),
                    'nb_schedule'        => $ech,
                    'reportDelay'        => $reportDelay,
                    'reportMode'         => $reportMode,
                    'modulePath'         => $this->module->modulePathFolder,
                    'eligibilityPath'    => Context::getContext()->link->getModuleLink($this->module->name,'eligibility', array(), true),
                    'token'              => Tools::getToken(),
                    'paymentCountryCode' => $paymentCountryCode,
                    'cgv_link'           => $this->module->getLinkByCountry($paymentCountryCode)
                );

                return $displayPayment;
            }
        }
    }

}