<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class floaCarrierMapping extends objectModel
{
    
    /** @var int primary ID */
    public $id;

    /** @var int id_carrier PS equivalent */
    public $ps_id;

    /** @var int FLOA carrier ID */
    public $cb4x_id;

    public static $definition = array(
        'table'          => 'floapay_carrier_mapping',
        'primary'        => 'id',
        'multilang'      => false,
        'multilang_shop' => false,
        'fields'         => array(
            'ps_id'   => array('type' => self::TYPE_INT),
            'floa_id' => array('type' => self::TYPE_STRING),
        )
    );
    
    /**
     * Module instance
     *
     * @var mixed
     */
    private $module;

    public function __construct()
    {
        $this->module = \Module::getInstanceByName("floapay");
        parent::__construct();
    }
    
    /**
     * getList
     *
     * @return array
     */
    public function getList()
    {
        return array(
            'CDS' => 'Colissimo direct',
            'CHR' => 'Chronopost',
            'COL' => 'Colissimo',
            'CRE' => 'Chronopost/Chronorelais',
            'KIA' => 'Kiala',
            'IMP' => 'Printed and sent to home',
            'LSP' => 'Delivery service plus',
            'MOR' => 'Mory',
            'TNT' => 'TNT',
            'TRP' => 'Transporter',
            'AE1' => 'Easydis error',
            'EA1' => 'Easydis',
            'KIB' => 'KIB',
            'TNB' => 'TNT Belgium',
            'EXP' => 'Express delivery',
            'AGE' => 'Agediss',
            'EMP' => 'Carried away',
            'M30' => 'Carried away -30',
            'ADX' => 'Adrexo',
            'EY1' => 'Carried away -30 Easydis',
            'VIR' => 'Virtuel',
            'REG' => 'Recommandé',
            'STD' => 'Normal',
            'TRK' => 'Followed',
            'PRM' => 'Premium Easydis',
            'RDO' => 'Confort (SOGEP) Easydis',
            'RCO' => 'Package relay (SOGEP) Cestas',
            'SO1' => 'SoColissimo overseas zone 1',
            'SO2' => 'SoColissimo overseas zone 2',
            'RIM' => 'Immediate pick up in store',
            'LDR' => 'LDR',
            'MAG' => 'Store delivery',
            'RDE' => 'Eco (SOGEP) Easydis',
            'REL' => 'Mondial Relay',
            'FDR' => 'Direct relay supplier',
            'TNX' => 'TNT express relay',
            'EMX' => 'Express',
            'CHX' => 'Carried away Chronopost relay',
            'CSX' => 'Carried away Chronopost deposit',
            'LDS' => 'Standard home delivery',
            'LDE' => 'Express home delivery',
            'LCS' => 'Standard center delivery',
            'LCE' => 'Express center delivery',
            'GLS' => 'GLS',
            'STU' => 'Stuart',
            'UPS' => 'UPS',
            'LCK' => 'Lockers',
            'DRO' => 'Drone delivery',
            'OT1' => 'Other shipping method #1',
            'OT2' => 'Other shipping method #2',
            'OT3' => 'Other shipping method #3'
        );
    }
    
    /**
     * getAll
     *
     * @return array
     */
    public function getAll()
    {
        $carriers   = \CarrierCore::getCarriers(
            \Context::getContext()->language->id,
            false, false, false, null, \CarrierCore::ALL_CARRIERS
        );
        $psCarriers = array();
        foreach ($carriers as $carrier) {
            $psCarriers[] = array(
                'ps_id' => $carrier['id_carrier'],
                'name'  => $carrier['name']
            );
        }

        return $psCarriers;
    }
    
    /**
     * getMappingAll
     *
     * @return array
     */
    public function getMappingAll()
    {
        $sql = 'SELECT
                    `' . 'ps_id' . '`,
                    `' . 'floa_id' . '`
                FROM `' . _DB_PREFIX_ . self::$definition['table'] . '`';

        $rows = \Db::getInstance()->ExecuteS($sql);

        return $rows
            ? array_combine(array_column($rows, 'ps_id'), array_column($rows, 'floa_id')) : array();
    }
    
    /**
     * getFloaIdByPsId
     *
     * @param  integer $psId
     * @return string
     */
    public function getFloaIdByPsId($psId)
    {
        $sql = 'SELECT `' . 'floa_id' . '`
                FROM `' . _DB_PREFIX_ . self::$definition['table'] . '`
                WHERE `' . 'ps_id' . '` = ' . (int)$psId;

        $row = \Db::getInstance()->getRow($sql);

        return $row ? $row['floa_id'] : 'STD';
    }
    
    /**
     * getIdByPsId
     *
     * @param  integer $psId
     * @return bool
     */
    public function getIdByPsId($psId)
    {
        $sql = 'SELECT `' . 'id' . '`
                FROM `' . _DB_PREFIX_ . self::$definition['table'] . '`
                WHERE `' . 'ps_id' . '` = ' . (int)$psId;

        return \Db::getInstance()->getValue($sql);
    }
}