<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class floaOrder extends objectModel
{
    
    /**
     * @var integer
     */
    public $merchant_id;

    /**
     * @var integer
     */
    public $merchant_site_id;

    /**
     * @var integer
     */
    public $id_cart;

    /**
     * @var integer
     */
    public $id_order;

    /**
     * @var date
     */
    public $scoring_date;

    /**
     * @var string
     */
    public $scoring_token;

    /**
     * @var string
     */
    public $payment_session_id;

    /**
     * @var integer
     */
    public $request_id;

    /**
     * @var date
     */
    public $validation_date;

    /**
     * @var date
     */
    public $payment_date;

    /**
     * @var date
     */
    public $cancel_date;

    /**
     * @var string
     */
    public $op;

    /**
     * @var string
     */
    public $order_reference;

    /**
     * @var integer
     */
    public $total_amount;

    /**
     * @var integer
     */
    public $fees_amount;

    /**
     * @var string
     */
    public $partial_cancels;

    public static $definition = array(
        'table'          => 'floapay_orders',
        'primary'        => 'id',
        'multilang'      => false,
        'multilang_shop' => false,
        'fields'         => array(
            'merchant_id'         => array('type' => self::TYPE_INT),
            'merchant_site_id'    => array('type' => self::TYPE_INT),
            'id_cart'             => array('type' => self::TYPE_INT),
            'id_order'            => array('type' => self::TYPE_INT),
            'scoring_date'        => array('type' => self::TYPE_DATE),
            'scoring_token'       => array('type' => self::TYPE_STRING),
            'payment_session_id'  => array('type' => self::TYPE_STRING),
            'request_id'          => array('type' => self::TYPE_INT),
            'validation_date'     => array('type' => self::TYPE_DATE),
            'payment_date'        => array('type' => self::TYPE_DATE),
            'cancel_date'         => array('type' => self::TYPE_DATE),
            'op'                  => array('type' => self::TYPE_STRING),
            'order_reference'     => array('type' => self::TYPE_STRING),
            'total_amount'        => array('type' => self::TYPE_INT),
            'fees_amount'         => array('type' => self::TYPE_INT),
            'partial_cancels'     => array('type' => self::TYPE_STRING)
        ),
    );
    
    /**
     * getByReference
     *
     * @param  string $reference
     * @return array
     */
    public static function getByReference($reference)
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . self::$definition['table'].'` WHERE `order_reference` = "' . pSQL($reference) . '"';
        $response = Db::getInstance()->getRow($sql);
        return $response;
    }
    
    /**
     * getByOrderId
     *
     * @param  integer $id
     * @return array
     */
    public static function getByOrderId($id)
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . self::$definition['table'].'` WHERE `id_order` = "' . (int) $id . '"';
        $response = Db::getInstance()->getRow($sql);
        return $response;
    }
    
    /**
     * updatePartialCancels
     *
     * @param  integer $id
     * @param  array $partial_cancels
     * @param  float $refund
     * @return bool
     */
    public static function updatePartialCancels($id, $partial_cancels, $refund)
    {
        $partialCancels = $partial_cancels == null ? array() : unserialize($partial_cancels);
        $partialCancels[] = array(
            'date'      => date('d/m/Y H:i:s'),
            'amount'    => Tools::ps_round(str_replace(",", "", $refund), 2)
        );
        $data   = array('partial_cancels' => pSQL(serialize($partialCancels)));
        $where  = 'id =' . (int)$id;
        return Db::getInstance()->update(self::$definition['table'], $data, $where);
    }

}