<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class floaTools
{
    CONST DOM_TOM_COUNTRIES = [
      'GP', 'MQ', 'GF', 'RE', 'PM', 'YT', 'BL', 'MF', 'WF', 'PF', 'NC', 'TF'
    ];
    
    /**
     * getUserIP
     *
     * @return string
     */
    public static function getUserIP() {
        $ipaddress = '';
        if (isset($_SERVER['HTTP_CLIENT_IP']))
            $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
        else if(isset($_SERVER['HTTP_X_FORWARDED_FOR']))
            $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
        else if(isset($_SERVER['HTTP_X_FORWARDED']))
            $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
        else if(isset($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']))
            $ipaddress = $_SERVER['HTTP_X_CLUSTER_CLIENT_IP'];
        else if(isset($_SERVER['HTTP_FORWARDED_FOR']))
            $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
        else if(isset($_SERVER['HTTP_FORWARDED']))
            $ipaddress = $_SERVER['HTTP_FORWARDED'];
        else if(isset($_SERVER['REMOTE_ADDR']))
            $ipaddress = $_SERVER['REMOTE_ADDR'];
        else
            $ipaddress = '';
        if($ipaddress == "::1") 
            $ipaddress = '';

        return self::formatIp($ipaddress);
    }

    public static function formatIp($ipaddress)
    {
        $ips = explode(',', $ipaddress);
        $long = ip2long($ips[0]);
        if (-1 == $long || !$long) {
            return '';
        }
        return $ips[0];
    }
    
    /**
     * generateOrderRef
     *
     * @param  string $method
     * @param  integer $id
     * @return string
     */
    public static function generateOrderRef($method, $id)
    {
        return $method.'_'.$id.'_D'.rand(0, 9999);
    }
    
    /**
     * convertToInt
     *
     * @param  float $amount
     * @return integer
     */
    public static function convertToInt($amount)
    {
        $finalAmount = $amount * 100;
        return intval(round($finalAmount));
    }
    
    /**
     * convertToFloat
     *
     * @param  integer $amount
     * @return float
     */
    public static function convertToFloat($amount)
    {
        $finalAmount = $amount / 100;
        return $finalAmount;
    }
    
    /**
     * getToken
     *
     * @return string
     */
    public static function getToken()
    {
        $token = sha1(mt_rand());
        Context::getContext()->cookie->__set('floaToken', $token);
        Context::getContext()->cookie->write();
        return $token;
    }
      
    /**
     * isTokenValid
     *
     * @param  string $token
     * @return bool
     */
    public static function isTokenValid($token)
    {
        if (Context::getContext()->cookie->__isset('floaToken')
            && Context::getContext()->cookie->__get('floaToken') == $token) {
            Context::getContext()->cookie->__unset('floaToken');
            return true;
        }
        return false;
    }

    /**
     * @param $country string
     * @return string
     */
    public static function getCountryCode($country)
    {
        $domTom = self::DOM_TOM_COUNTRIES;
        return (in_array($country, $domTom)) ? 'FR' : $country;
    }

     /**
     * computeCulture
     *
     * @param  string $country
     * @return mixed
     */
    public static function computeCulture($country)
    {
        $domTom = self::DOM_TOM_COUNTRIES;
        $country = (in_array($country, $domTom)) ? 'FR' : $country;
        $language_code  = explode('-', Context::getContext()->language->language_code);
        return Tools::strtolower($language_code[0]).'-'.Tools::strtoupper($country);
    }
}