/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

 var checkDateValidity = function (value) {
    var regex = /^((((19|[2-9]\d)\d{2})\-(0[13578]|1[02])\-(0[1-9]|[12]\d|3[01]))|(((19|[2-9]\d)\d{2})\-(0[13456789]|1[012])\-(0[1-9]|[12]\d|30))|(((19|[2-9]\d)\d{2})\-02\-(0[1-9]|1\d|2[0-8]))|(((1[6-9]|[2-9]\d)(0[48]|[2468][048]|[13579][26])|((16|[2468][048]|[3579][26])00))\-02\-29))$/;
    return regex.test(value);
};

var checkDateMajorValidity = function (value) {
    dt1 = new Date(value);
    dt2 = new Date();
    var diff =(dt2.getTime() - dt1.getTime()) / 1000;
    diff /= (60 * 60 * 24);
    var counterDiff =  Math.abs(Math.round(diff/365.25));
    return counterDiff >= 18 ? true : false;
};

var checkMaidennameValidity = function (value) {
    var regex = /^[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]+(([',. -][a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ ])?[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]*)*$/;
    return regex.test(value);
};

var checkCpValidity = function (value, country) {
    switch (country) {
        case 'FR' :
        default :
            var regex = /^\d{2}|^\d{3}$|^\d{5}$/;
        break;
        case 'IT' :
            var regex = /^99$|^\d{3}$|^\d{5}$/;
            break;
        case 'BE' :
            var regex = /^99$|^\d{4}$/;
            break;
        case 'PT' :
            var regex = /^99$|^\d{4}-\d{3}$/;
            break;
        case 'ES' :
            var regex = /^99$|^\d{5}$/;
            break;
    }
    return regex.test(value);
};

var checkPhoneValidity = function (value, country) {
    switch (country) {
        case 'FR' :
        default :
            var regex = /^((\+)33|0|0033)[1-9](\d{2}){4}$/;
            break;
        case 'IT' :
            var regex = /^((?:0039)?|(?:\+39)?)3[0-9]{8,9}$/;
            break;
        case 'BE' : 
            var regex = /^((\+|00)32\s?|0)4(\d{2})(\s?\d{2}){3}$/;
            break;
        case 'PT' : 
            var regex = /^((\+351|00351|351)?)(2\d{1}|(9(3|6|2|1)))\d{7}$/;
            break;
        case 'ES' : 
            var regex = /^(([+]|00)34)?(?:[0-9]{8,20})$/;
        break;
    } 
    return regex.test(value);
};

var checkNifValidity = function (value, country) {
    switch (country) {
        case 'IT' :
            var regex = /^([a-zA-Z]{6})+([0-9]{2})+([a-zA-Z]{1})+([0-9]{2})+([a-zA-Z]{1})+([0-9]{3})+([a-zA-Z]{1})$/;
            break;
        case 'PT' :
            var regex = /^[0-9]{9}$/;
            break;
        case 'ES' :
            var regex = /^[0-9]{8}[A-Z]{1}$|^[ILMOTXYZ][0-9]{7}[A-Z]{1}$/;
            break;
        }
    return regex.test(value);
}

var checkSecondLastNameValidity = function (value) {
    var regex = /^[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]+(([',. -][a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ ])?[a-zA-Zàáâãäåçèéêëìíîïðòóôõöùúûüýÿ]*)*$/;
    return regex.test(value);
}

var floaEligibilityError = function (context, selectedPaymentMethod, errors) {
    if (context == "displayPayment") {
        var html_error = '<div class="errorFormContainer alert alert-danger mb-0"><ul class="mb-0"></ul></div>';
        $('.selected_instance_'+selectedPaymentMethod+' #errorForm_'+selectedPaymentMethod).html(html_error);
        if (errors.length > 0) {
            errors.forEach(function (element) {
                $('.selected_instance_'+selectedPaymentMethod+' #errorForm_'+selectedPaymentMethod+' ul').append('<li>' + element + '</li>');
            });    
        } else {
            $('.selected_instance_'+selectedPaymentMethod+' #errorForm_'+selectedPaymentMethod+' ul').append('<li>' + _ERROR_OCCURED_ + '</li>');
        }
    } else {
        var html_error = '<div class="errorFormContainer alert alert-danger mb-0"><ul class="mb-0"></ul></div>';
        $('#errorForm_'+selectedPaymentMethod).html(html_error);
        if (errors.length > 0) {
            errors.forEach(function (element) {
                $('#errorForm_'+selectedPaymentMethod+' ul').append('<li>' + element + '</li>');
            });    
        } else {
            $('#errorForm_'+selectedPaymentMethod+' ul').append('<li>' + _ERROR_OCCURED_ + '</li>');
        }
    }
};

var floaValidatorForm = function (datas, country) {
    var error_message = [];
    var validator = true;
    if (datas.ReportMode == 1 && !checkDateValidity(datas.ReportDate) && datas.PaymentMethod == "CB1XD") {
        error_message.push(_PAYMENT_DATE_SCORE_INVALID_);
        validator = false;
    }
    if (datas.Civility != 1 && datas.Civility != 2) {
        error_message.push(_CIVILITY_SCORE_INVALID_);
        validator = false;
    }
    if (datas.Civility == 2 && !checkMaidennameValidity(datas.MaidenName) 
        && (country == "FR" || country == "BE")) {
        error_message.push(_MAIDEN_SCORE_INVALID_);
        validator = false;
    }
    if (country == "ES" && !checkSecondLastNameValidity(datas.SecondLastName)) {
        error_message.push(_SECOND_LAST_NAME_INVALID_);
        validator = false;
    }
    if(country == "IT" || country == "PT" || country == "ES") {
        if (!checkNifValidity(datas.Nif, country)) {
            error_message.push(_NIF_SCORE_INVALID_);
            validator = false;
        }    
    }
    if (!checkPhoneValidity(datas.CellPhone, country)) {
        error_message.push(_PHONE_SCORE_INVALID_);
        validator = false;
    }
    if (!checkCpValidity(datas.PostalCode, country)) {
        error_message.push(_CP_SCORE_INVALID_);
        validator = false;
    }
    if (!checkDateValidity(datas.BirthDate)) {
        error_message.push(_DDN_SCORE_INVALID_);
        validator = false;
    } else if (!checkDateMajorValidity(datas.BirthDate)) {
        error_message.push(_DDN_SCORE_MAJOR_INVALID_);
        validator = false;
    }
    return {
        'state'    : validator,
        'messages' : error_message
    }
}

var ComputeFloaFormDatas = function (selectedPaymentMethod, formScoring) {
    return datas = {
        'ReportMode'    : $("#reportMode").val(),
        'ReportDate'    : (selectedPaymentMethod == "CB1XD" ? $(formScoring + ' input[name=payment_date]').val() : false),
        'Civility'      : $(formScoring + ' select[name=score_civility]').val(),
        'Nif'           : $(formScoring + ' input[name=score_nif]').val(),
        'BirthDate'     : $(formScoring + ' input[name=score_ddn]').val(),
        'MaidenName'    : $(formScoring + ' input[name=score_maidenname]').val(),
        'PostalCode'    : $(formScoring + ' input[name=score_cp]').val(),
        'CellPhone'     : $(formScoring + ' input[name=score_phone]').val(),
        'PaymentMethod' : selectedPaymentMethod
    };
}

$(function(){
    $(document).on('change', '.civility_choice', function(){
        if (_FLOA_COUNTRY_CONTEXT_ == "FR" || _FLOA_COUNTRY_CONTEXT_ == "BE") {
            var id = $(this).data('id');
            if ($(this).val() == 2) {
                $('.maiden_name_'+id).show();
            } else {
                $('.maiden_name_'+id).hide();
            }    
        }
    });
});
