{*
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 *}
 
<div class="card mt-2 d-print-none panel">
    <div class="card-header">
        <div class="row">
            <div class="col-md-12">
                <h3 class="card-header-title panel-heading">
                    <img src="{$module_icon_path|escape:'htmlall':'UTF-8'}floa_{$orderDetails.op|escape:'htmlall':'UTF-8'}_tiny.png" /> FloaPay
                </h3>
            </div>
        </div>
    </div>
    <div class="card-body">
    {if isset($errorMessage) && $errorMessage != ''}
        <div class="alert alert-danger" data-rounding-alert=""><p>{$errorMessage|escape:'htmlall':'UTF-8'}</p></div> 
    {elseif isset($successMessage) && $successMessage != ''}
        <div class="alert alert-success" data-rounding-alert=""><p>{$successMessage|escape:'htmlall':'UTF-8'}</p></div>
    {/if}
    {if isset($apiState) && $apiState == true}
        {if $schedules[0]->state == "init" && $orderDetails.op != "CB1XD" && $orderDetails.op != "CB10X"}
            <p style="color:red;">{l s='PLEASE NOTE, payment capture has not yet been performed for this order.' mod='floapay'}</p>
           <form method="post" id="floaPayForm" class="floaPayForm">
                <input type="hidden" name="floaToken" id="floaToken" value="{$floaToken|escape:'htmlall':'UTF-8'}" />
                <input type="hidden" name="floaOrderAction" id="floaOrderActionPay" value="pay" />
                <input class="mb-2 btn btn-primary" type="submit" value="{l s='Capture order' mod='floapay'}" />
            </form>
        {/if}
        <dl>
            <dt>{l s='Order reference' mod='floapay'} : <strong>{$orderDetails.order_reference|escape:'htmlall':'UTF-8'}</strong></dt>
            <dt>{l s='Order amount' mod='floapay'} : <strong>{Tools::displayPrice($orderWithoutFees/100)|escape:'htmlall':'UTF-8'}</strong></dt>
            <dt>{l s='Financing fees' mod='floapay'} : <strong>{Tools::displayPrice($orderDetails.fees_amount/100)|escape:'htmlall':'UTF-8'}</strong></dt>
            <dt>{l s='Scoring date' mod='floapay'} : <strong>{if $orderDetails.scoring_date != '0000-00-00 00:00:00'}{date('d/m/Y H:i:s', strtotime($orderDetails.scoring_date|escape:'htmlall':'UTF-8'))}{/if}</strong></dt>
            <dt>{l s='Payment date' mod='floapay'} : <strong>{if $orderDetails.payment_date != '0000-00-00 00:00:00'}{date('d/m/Y H:i:s', strtotime($orderDetails.payment_date|escape:'htmlall':'UTF-8'))}{/if}</strong></dt>
        </dl>
        <p><strong>{l s='Schedules' mod='floapay'}</strong></p>
        <table class="table" width="100%" cellspacing="0" cellpadding="0">
            <thead>
                <tr>
                    <th>{l s='Rank' mod='floapay'}</th>
                    <th>{l s='Date' mod='floapay'}</th>
                    <th>{l s='Price' mod='floapay'}</th>
                    <th>{l s='State' mod='floapay'}</th>
                </tr>
            </thead>
            <tbody>
           {if $schedules}
                {foreach from=$schedules item=schedule}
                    <tr>
                        <td>{l s='Rank' mod='floapay'} {$schedule->rank|escape:'htmlall':'UTF-8'}</td>
                        <td>{date('d/m/Y', strtotime($schedule->date|escape:'htmlall':'UTF-8'))}</td>
                        <td>{Tools::displayPrice($schedule->amount|escape:'htmlall':'UTF-8'/100)}</td>
                        <td>{$schedule->state|escape:'htmlall':'UTF-8'}</td>
                    </tr>
                {/foreach}
            {/if}
            </tbody>
        </table>
        <br/>
        {if $orderRefunds != null}
            <p><strong>{l s='Refunds' mod='floapay'}</strong></p>
            <table class="table" width="100%" cellspacing="0" cellpadding="0">
                <thead>
                    <tr>
                        <th>{l s='Date' mod='floapay'}</th>
                        <th>{l s='Refunded amount' mod='floapay'}</th>
                    </tr>
                </thead>
                <tbody>
                    {foreach from=$orderRefunds item=orderRefund}
                        <tr>
                            <td>{$orderRefund.date|escape:'htmlall':'UTF-8'}</td>
                            <td>{Tools::displayPrice($orderRefund.amount/100)|escape:'htmlall':'UTF-8'}</td>
                        </tr>
                    {/foreach}
                </tbody>
            </table>
            <br/>
        {/if}
        {if $orderWithoutFees > 0}
            <div class="floa-refund-form">
                <p><strong>{l s='Make a refund' mod='floapay'}</strong></p>
                <hr />
                <i>{l s='Maximum refund amount %s' sprintf={Tools::displayPrice(($orderWithoutFees/100)|escape:'htmlall':'UTF-8')} mod='floapay'}</i>
                <br />
                <br />
                <form method="post" id="floaRefundForm" class="floaRefundForm">
                    <input type="hidden" name="floaOrderAction" id="floaOrderActionRefund" value="refund" />
                    <input type="hidden" name="floaToken" id="floaToken" value="{$floaToken|escape:'htmlall':'UTF-8'}" />
                    <span>{l s='Amount to refund' mod='floapay'} : </span>
                    <div class="form-group"><input type="text" name="floaRefundAmount" id="floaRefundAmount" class="refundAmount" value="" required="required"></div>
                    <input class="btn btn-primary mt-1" type="submit" value="{l s='refund' mod='floapay'}" />
                </form>
            </div>
        {/if}
        <script type="text/javascript">
            var _CONFIRM_REFUND_ = "{l s='Please confirm the refund' mod='floapay'}";
            var _CONFIRM_PAY_ = "{l s='Please confirm the capture' mod='floapay'}";
        </script>
    {else}
        <div>{l s='Unable to retrieve order information.' mod='floapay'}</div>
    {/if}

    </div>
</div>
<script type="text/javascript">
    $(function(){
        $('.refundAmount').on('input', function() {
            this.value = this.value.replace(/[^0-9.]/g, '').replace(/(\..*)\./g, '$1');
        });    
    });
    $(function(){
        $('.floaRefundForm').on('submit', function() {
            if (!confirm(_CONFIRM_REFUND_)) {
                return false;
            } else {
                return true;
            }
        });
        $('.floaPayForm').on('submit', function() {
            if (!confirm(_CONFIRM_PAY_)) {
                return false;
            } else {
                return true;
            }
        });
    });
</script>